@@ -31,8 +31,17 @@ error "gperf generated tables don't work with this execution character set. Plea
 
 #line 1 "atoms.gperf"
 
+
+/* Rely on vasprintf (GNU extension) instead of vsnprintf if
+   possible... */
+#ifdef HAVE_VASPRINTF
+#define _GNU_SOURCE
+#include <stdio.h>
+#endif
+
 #include <xcb/xcb.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include "xcb_atom.h"
 
 const xcb_atom_t PRIMARY = 1;
@@ -103,7 +112,7 @@ const xcb_atom_t FULL_NAME = 65;
 const xcb_atom_t CAP_HEIGHT = 66;
 const xcb_atom_t WM_CLASS = 67;
 const xcb_atom_t WM_TRANSIENT_FOR = 68;
-#line 84 "atoms.gperf"
+#line 93 "atoms.gperf"
 struct atom_map { int name; xcb_atom_t value; };
 #include <string.h>
 /* maximum key range = 146, duplicates = 0 */
@@ -336,171 +345,171 @@ in_word_set (str, len)
   static const struct atom_map wordlist[] =
     {
       {-1}, {-1}, {-1}, {-1},
-#line 103 "atoms.gperf"
+#line 112 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str4,18},
       {-1},
-#line 147 "atoms.gperf"
+#line 156 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str6,62},
       {-1}, {-1},
-#line 148 "atoms.gperf"
+#line 157 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str9,63},
-#line 106 "atoms.gperf"
+#line 115 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str10,21},
-#line 143 "atoms.gperf"
+#line 152 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str11,58},
       {-1}, {-1},
-#line 128 "atoms.gperf"
+#line 137 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str14,43},
-#line 151 "atoms.gperf"
+#line 160 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str15,66},
-#line 90 "atoms.gperf"
+#line 99 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str16,5},
-#line 104 "atoms.gperf"
+#line 113 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str17,19},
       {-1},
-#line 146 "atoms.gperf"
+#line 155 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str19,61},
       {-1}, {-1}, {-1}, {-1},
-#line 150 "atoms.gperf"
+#line 159 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str24,65},
       {-1},
-#line 138 "atoms.gperf"
+#line 147 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str26,53},
-#line 139 "atoms.gperf"
+#line 148 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str27,54},
-#line 114 "atoms.gperf"
+#line 123 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str28,29},
-#line 131 "atoms.gperf"
+#line 140 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str29,46},
       {-1}, {-1},
-#line 110 "atoms.gperf"
+#line 119 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str32,25},
-#line 91 "atoms.gperf"
+#line 100 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str33,6},
       {-1}, {-1},
-#line 93 "atoms.gperf"
+#line 102 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str36,8},
-#line 121 "atoms.gperf"
+#line 130 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str37,36},
-#line 120 "atoms.gperf"
+#line 129 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str38,35},
       {-1}, {-1},
-#line 101 "atoms.gperf"
+#line 110 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str41,16},
-#line 113 "atoms.gperf"
+#line 122 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str42,28},
-#line 102 "atoms.gperf"
+#line 111 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str43,17},
       {-1},
-#line 112 "atoms.gperf"
+#line 121 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str45,27},
-#line 118 "atoms.gperf"
+#line 127 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str46,33},
-#line 111 "atoms.gperf"
+#line 120 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str47,26},
-#line 136 "atoms.gperf"
+#line 145 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str48,51},
       {-1}, {-1},
-#line 115 "atoms.gperf"
+#line 124 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str51,30},
       {-1},
-#line 117 "atoms.gperf"
+#line 126 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str53,32},
-#line 107 "atoms.gperf"
+#line 116 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str54,22},
       {-1},
-#line 100 "atoms.gperf"
+#line 109 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str56,15},
-#line 124 "atoms.gperf"
+#line 133 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str57,39},
-#line 141 "atoms.gperf"
+#line 150 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str58,56},
       {-1}, {-1},
-#line 135 "atoms.gperf"
+#line 144 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str61,50},
-#line 86 "atoms.gperf"
+#line 95 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str62,1},
-#line 92 "atoms.gperf"
+#line 101 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str63,7},
-#line 137 "atoms.gperf"
+#line 146 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str64,52},
-#line 142 "atoms.gperf"
+#line 151 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str65,57},
-#line 108 "atoms.gperf"
+#line 117 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str66,23},
-#line 122 "atoms.gperf"
+#line 131 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str67,37},
-#line 109 "atoms.gperf"
+#line 118 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str68,24},
       {-1},
-#line 125 "atoms.gperf"
+#line 134 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str70,40},
-#line 99 "atoms.gperf"
+#line 108 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str71,14},
       {-1},
-#line 152 "atoms.gperf"
+#line 161 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str73,67},
       {-1},
-#line 119 "atoms.gperf"
+#line 128 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str75,34},
-#line 98 "atoms.gperf"
+#line 107 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str76,13},
       {-1},
-#line 133 "atoms.gperf"
+#line 142 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str78,48},
-#line 89 "atoms.gperf"
+#line 98 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str79,4},
-#line 129 "atoms.gperf"
+#line 138 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str80,44},
-#line 153 "atoms.gperf"
+#line 162 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str81,68},
-#line 123 "atoms.gperf"
+#line 132 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str82,38},
-#line 127 "atoms.gperf"
+#line 136 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str83,42},
-#line 130 "atoms.gperf"
+#line 139 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str84,45},
-#line 144 "atoms.gperf"
+#line 153 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str85,59},
-#line 105 "atoms.gperf"
+#line 114 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str86,20},
       {-1}, {-1}, {-1},
-#line 145 "atoms.gperf"
+#line 154 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str90,60},
-#line 134 "atoms.gperf"
+#line 143 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str91,49},
-#line 140 "atoms.gperf"
+#line 149 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str92,55},
-#line 88 "atoms.gperf"
+#line 97 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str93,3},
       {-1}, {-1},
-#line 97 "atoms.gperf"
+#line 106 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str96,12},
       {-1},
-#line 126 "atoms.gperf"
+#line 135 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str98,41},
       {-1}, {-1},
-#line 96 "atoms.gperf"
+#line 105 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str101,11},
       {-1}, {-1}, {-1}, {-1},
-#line 95 "atoms.gperf"
+#line 104 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str106,10},
       {-1},
-#line 132 "atoms.gperf"
+#line 141 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str108,47},
       {-1}, {-1},
-#line 94 "atoms.gperf"
+#line 103 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str111,9},
       {-1}, {-1}, {-1}, {-1},
-#line 116 "atoms.gperf"
+#line 125 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str116,31},
       {-1}, {-1}, {-1}, {-1},
-#line 149 "atoms.gperf"
+#line 158 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str121,64},
       {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
       {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
       {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
-#line 87 "atoms.gperf"
+#line 96 "atoms.gperf"
       {(int)(long)&((struct stringpool_t *)0)->stringpool_str149,2}
     };
 
@@ -522,7 +531,7 @@ in_word_set (str, len)
     }
   return 0;
 }
-#line 154 "atoms.gperf"
+#line 163 "atoms.gperf"
 
 
 static const char atom_names[] =
@@ -667,7 +676,7 @@ static const uint16_t atom_name_offsets[] = {
 	749,
 };
 
-xcb_atom_t intern_atom_predefined(uint16_t name_len, const char *name)
+xcb_atom_t xcb_atom_get_predefined(uint16_t name_len, const char *name)
 {
 	const struct atom_map *value = in_word_set(name, name_len);
 	xcb_atom_t ret = XCB_NONE;
@@ -676,11 +685,11 @@ xcb_atom_t intern_atom_predefined(uint16_t name_len, const char *name)
 	return ret;
 }
 
-intern_atom_fast_cookie_t intern_atom_fast(xcb_connection_t *c, uint8_t only_if_exists, uint16_t name_len, const char *name)
+xcb_atom_fast_cookie_t xcb_atom_get_fast(xcb_connection_t *c, uint8_t only_if_exists, uint16_t name_len, const char *name)
 {
-	intern_atom_fast_cookie_t cookie;
+	xcb_atom_fast_cookie_t cookie;
 
-	if((cookie.u.atom = intern_atom_predefined(name_len, name)) != XCB_NONE)
+	if((cookie.u.atom = xcb_atom_get_predefined(name_len, name)) != XCB_NONE)
 	{
 		cookie.tag = TAG_VALUE;
 		return cookie;
@@ -691,7 +700,7 @@ intern_atom_fast_cookie_t intern_atom_fast(xcb_connection_t *c, uint8_t only_if_
 	return cookie;
 }
 
-xcb_atom_t intern_atom_fast_reply(xcb_connection_t *c, intern_atom_fast_cookie_t cookie, xcb_generic_error_t **e)
+xcb_atom_t xcb_atom_get_fast_reply(xcb_connection_t *c, xcb_atom_fast_cookie_t cookie, xcb_generic_error_t **e)
 {
 	switch(cookie.tag)
 	{
@@ -714,17 +723,17 @@ xcb_atom_t intern_atom_fast_reply(xcb_connection_t *c, intern_atom_fast_cookie_t
 	return cookie.u.atom;
 }
 
-const char *get_atom_name_predefined(xcb_atom_t atom)
+const char *xcb_atom_get_name_predefined(xcb_atom_t atom)
 {
 	if(atom <= 0 || atom > (sizeof(atom_name_offsets) / sizeof(*atom_name_offsets)))
 		return 0;
 	return atom_names + atom_name_offsets[atom - 1];
 }
 
-int get_atom_name(xcb_connection_t *c, xcb_atom_t atom, const char **namep, int *lengthp)
+int xcb_atom_get_name(xcb_connection_t *c, xcb_atom_t atom, const char **namep, int *lengthp)
 {
 	static char buf[100];
-	const char *name = get_atom_name_predefined(atom);
+	const char *name = xcb_atom_get_name_predefined(atom);
 	int namelen;
 	xcb_get_atom_name_cookie_t atomc;
 	xcb_get_atom_name_reply_t *atomr;
@@ -747,3 +756,68 @@ int get_atom_name(xcb_connection_t *c, xcb_atom_t atom, const char **namep, int
 	free(atomr);
 	return 1;
 }
+
+static char *makename(const char *fmt, ...)
+{
+	char *ret;
+	int n;
+	va_list ap;
+
+#ifndef HAVE_VASPRINTF
+	char *np;
+	int size = 64;
+
+	/* First allocate 'size' bytes, should be enough usually */
+	if((ret = malloc(size)) == NULL)
+		return NULL;
+
+	while(1)
+	{
+		va_start(ap, fmt);
+		n = vsnprintf(ret, size, fmt, ap);
+		va_end(ap);
+
+		if(n < 0)
+			return NULL;
+
+		if(n < size)
+			return ret;
+
+		size = n + 1;
+		if((np = realloc(ret, size)) == NULL)
+		{
+			free(ret);
+			return NULL;
+		}
+
+		ret = np;
+	}
+#else
+	va_start(ap, fmt);
+	n = vasprintf(&ret, fmt, ap);
+	va_end(ap);
+
+	if(n < 0)
+		return NULL;
+
+	return ret;
+#endif
+}
+
+char *xcb_atom_name_by_screen(const char *base, uint8_t screen)
+{
+	return makename("%s_S%u", base, screen);
+}
+
+char *xcb_atom_name_by_resource(const char *base, uint32_t resource)
+{
+	return makename("%s_R%08X", base, resource);
+}
+
+char *xcb_atom_name_unique(const char *base, uint32_t id)
+{
+	if(base)
+		return makename("%s_U%lu", base, id);
+	else
+		return makename("U%lu", id);
+}
